@@ -6,30 +6,29 @@
 
 #include <ATen/ATen.h>
 #include <ATen/hip/impl/HIPStreamMasqueradingAsCUDA.h>
-#include <c10/util/Optional.h>
+#include <optional>
 
 #include <cstddef>
 #include <vector>
 
-namespace torch {
-namespace autograd {
+namespace torch::autograd {
 
 struct TORCH_CUDA_CU_API Scatter : public Node {
   explicit Scatter(
       std::vector<at::Device> devices,
-      c10::optional<std::vector<int64_t>> chunk_sizes = c10::nullopt,
+      std::optional<std::vector<int64_t>> chunk_sizes = std::nullopt,
       int64_t dim = 0,
-      c10::optional<std::vector<c10::optional<at::hip::HIPStreamMasqueradingAsCUDA>>> streams =
-          c10::nullopt,
+      std::optional<std::vector<std::optional<at::hip::HIPStreamMasqueradingAsCUDA>>> streams =
+          std::nullopt,
       bool unsqueeze_scalars = false);
   ~Scatter() override;
 
   variable_list apply(variable_list&& inputs) override;
 
   std::vector<at::Device> devices_;
-  c10::optional<std::vector<int64_t>> chunk_sizes_;
+  std::optional<std::vector<int64_t>> chunk_sizes_;
   int64_t dim_;
-  c10::optional<std::vector<c10::optional<at::hip::HIPStreamMasqueradingAsCUDA>>> streams_;
+  std::optional<std::vector<std::optional<at::hip::HIPStreamMasqueradingAsCUDA>>> streams_;
   bool unsqueeze_scalars_;
 };
 
@@ -43,5 +42,4 @@ struct TORCH_CUDA_CU_API Gather : public Node {
   int64_t dim_;
 };
 
-} // namespace autograd
-} // namespace torch
+} // namespace torch::autograd
